@@ -798,7 +798,7 @@ subst_variables(const gchar* src)
 
               if ((retval_len - pos) < varval_len)
                 {
-                  retval_len *= 2;
+                  retval_len = pos + varval_len;
                   retval = g_realloc(retval, retval_len+3);
                 }
               
@@ -1067,6 +1067,7 @@ void
 gconf_log(GConfLogPriority pri, const gchar* fmt, ...)
 {
   gchar* msg;
+  gchar* convmsg;
   va_list args;
   int syslog_pri = LOG_DEBUG;
 
@@ -1119,7 +1120,14 @@ gconf_log(GConfLogPriority pri, const gchar* fmt, ...)
           break;
         }
 
-      syslog(syslog_pri, "%s", msg);
+      convmsg = g_locale_from_utf8 (msg, -1, NULL, NULL, NULL);
+      if (!convmsg)
+        syslog (syslog_pri, "%s", msg);
+      else
+        {
+	  syslog (syslog_pri, "%s", convmsg);
+	  g_free (convmsg);
+	}
     }
   else
     {
@@ -1130,13 +1138,13 @@ gconf_log(GConfLogPriority pri, const gchar* fmt, ...)
         case GCL_CRIT:
         case GCL_ERR:
         case GCL_WARNING:
-          fprintf(stderr, "%s\n", msg);
+          g_printerr ("%s\n", msg);
           break;
       
         case GCL_NOTICE:
         case GCL_INFO:
         case GCL_DEBUG:
-          printf("%s\n", msg);
+          g_print ("%s\n", msg);
           break;
 
         default:
@@ -1355,7 +1363,7 @@ gconf_value_list_to_primitive_list_destructive(GConfValue* val,
         *err = gconf_error_new(GCONF_ERROR_TYPE_MISMATCH,
                                _("Expected list of %s, got list of %s"),
                                gconf_value_type_to_string(list_type),
-                               gconf_value_type_to_string(val->type));
+                               gconf_value_type_to_string(gconf_value_get_list_type (val)));
       gconf_value_free(val);
       return NULL;
     }
@@ -2414,7 +2422,7 @@ read_current_server_and_set_warning (const gchar *iorfile,
        * or <pid>:none for gconftool
        */
       str = buf;
-      while (isdigit(*str))
+      while (isdigit ((unsigned char) *str))
         ++str;
 
       if (*str == ':')
@@ -2701,6 +2709,24 @@ gconf_get_current_lock_holder  (const gchar *lock_directory,
   return server;
 }
 
+void
+gconf_daemon_blow_away_locks (void)
+{
+  char *lock_directory;
+  char *iorfile;
+  
+  lock_directory = gconf_get_lock_dir ();
+
+  iorfile = g_strconcat (lock_directory, "/ior", NULL);
+
+  if (unlink (iorfile) < 0)
+    g_printerr (_("Failed to unlink lock file %s: %s\n"),
+                iorfile, g_strerror (errno));
+
+  g_free (iorfile);
+  g_free (lock_directory);
+}
+
 static CORBA_ORB gconf_orb = CORBA_OBJECT_NIL;      
 
 CORBA_ORB
@@ -2754,8 +2780,22 @@ gconf_orb_release (void)
 
 char*
 gconf_get_daemon_dir (void)
-{
-  return g_strconcat (g_get_home_dir (), "/.gconfd", NULL);
+{  
+  if (gconf_use_local_locks ())
+    {
+      char *s;
+      char *subdir;
+
+      subdir = g_strconcat ("gconfd-", g_get_user_name (), NULL);
+      
+      s = g_build_filename (g_get_tmp_dir (), subdir, NULL);
+
+      g_free (subdir);
+
+      return s;
+    }
+  else
+    return g_strconcat (g_get_home_dir (), "/.gconfd", NULL);
 }
 
 char*
@@ -2959,3 +2999,24 @@ _gconf_init_i18n (void)
       done = TRUE;
     }
 }
+
+enum { UNKNOWN, LOCAL, NORMAL };
+
+gboolean
+gconf_use_local_locks (void)
+{
+  static int local_locks = UNKNOWN;
+  
+  if (local_locks == UNKNOWN)
+    {
+      const char *l =
+        g_getenv ("GCONF_GLOBAL_LOCKS");
+
+      if (l && atoi (l) == 1)
+        local_locks = NORMAL;
+      else
+        local_locks = LOCAL;
+    }
+
+  return local_locks == LOCAL;
+}
